<div class="table-responsive-sm">
    <table data-page-list="[10, 25, All]" data-pagination="true" data-toggle="table" data-search="true" class="table-sm" style="font-size:13px; width:100%;">
    <thead class="thead-light" style="font-size:13px">
        <tr>
            <th class="text-nowrap text-center">Sample name</th>
            <th class="text-nowrap text-center">Status</th>
            <th class="text-nowrap text-center">Runtime</th>
            <th class="text-nowrap text-center">Peak memory use</th>
        </tr>
    </thead>
        <tbody>
            {% for element in sample_link_names %}
                {% set i = loop.index - 1 %}
                <tr>
                    <td>
                        <b>{{ sample_link_names[i] }}</b>
                    </td>
                    <td class='{{ row_classes[i] }}'>
                        {{ flags[i] }}
                    </td>
                    <td>
                        {{ times[i] }}
                    </td>
                    <td>
                        {{ mems[i] }}
                    </td>
                </tr>
            {% endfor %}
            </tr>
        </tbody>
    </table>
</div>
<style>
    div.fixed-table-pagination {
      font-size: 11px !important;
    }
    .search-input {
        text-align: center;
        font-size: 11px !important;
        width: 70px !important;
        height: calc(1.5rem + 2px) !important;
    }
    .float-right.search.btn-group {
        margin-top: 0px !important;
    }
    .btn-secondary.dropdown-toggle {
        font-size: 0.6rem !important;
        padding: 0.25rem !important;
    }
    .page-list {
        padding-right: 10px !important;
    }
</style>
<script src="http://code.jquery.com/jquery-latest.min.js" type="text/javascript"></script>
<script src="https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.14.6/umd/popper.min.js" integrity="sha384-wHAiFfRlMFy6i5SRaxvfOCifBUQy1xHdJ/yoi7FRNXMRBu5WHdZYu1hA6ZOblgut" crossorigin="anonymous"></script>
<script src="https://stackpath.bootstrapcdn.com/bootstrap/4.1.3/js/bootstrap.min.js" integrity="sha384-ChfqqxuZUCnJSK3+MXmPNIyE6ZbWh2IMqE241rYiqJxyMiZ6OW/JmZQ5stwEULTy" crossorigin="anonymous"></script>
<link href="https://unpkg.com/bootstrap-table@1.15.3/dist/bootstrap-table.min.css" rel="stylesheet">
<script src="https://unpkg.com/bootstrap-table@1.15.3/dist/bootstrap-table.min.js"></script>
