<!doctype html>
<html lang="en">
    <head>
      {% include "head.html" %}
      <title>Looper: status</title>
    </head>
    {{ navbar }}
    <body>
	<div class="container">
		<div class="container-fluid">
			<h3>Status by sample</h3>
		</div>
			{{ status_table }}
	</div>
    </body>
    <hr class="featurette-divider">
	<footer class="text-muted">
        {{ footer }}
    </footer>
</html>
