        <div id="top"></div>
        <nav class="navbar sticky-top navbar-expand-md navbar-dark bg-dark">
          <a class="navbar-left" href="{{ index_html }}" >{% include 'logo.html' %}</a>
          <button class="navbar-toggler" type="button" data-toggle="collapse" data-target="#navbarSupportedContent" aria-controls="navbarSupportedContent" aria-expanded="false" aria-label="Toggle navigation">
            <span class="navbar-toggler-icon"></span>
          </button>
          <div class="collapse navbar-collapse" id="navbarSupportedContent">
            <ul class="navbar-nav me-auto mt-2 mt-lg-0">
              {{ navbar_links }}
            </ul>
            <ul class="nav navbar-nav navbar-right">
              <a class="nav-link left" href="http://looper.databio.org/" target="_blank"><i class="fa fa-2x fa-book" aria-hidden="true"></i></a>
              <a class="nav-link left" href="https://github.com/pepkit/looper" target="_blank"><i class="fa fa-2x fa-github" aria-hidden="true"></i></a>
            </ul>
          </div>
        </nav>
